/*
 * $Id: MailClient.java,v 1.3 2005/02/10 16:28:32 suhrin Exp $
 *
 * Copyright (c) 1999-2005 Gnome Ltd. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Gnome Ltd. You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Gnome Ltd.
 */

package SK.gnome.dwarf.mail.sample;

import java.io.*;
import java.util.Properties;

import javax.security.auth.Subject;

import SK.gnome.dwarf.*;
import SK.gnome.dwarf.main.*;
import SK.gnome.dwarf.mail.mime.MimePartBuilder;
import SK.gnome.dwarf.mail.mime.MimeMessageBuilder;
import SK.gnome.dwarf.mail.smtp.SMTPServer;

/**
 * Sample command for sending test messages via local server.
 *
 */

public class MailClient extends Command
{ private SMTPServer server;
  private MimeMessageBuilder msg;

  /**
   * Creates a new <tt>MailClient</tt>.
   */

  public MailClient(String name)
  { super(name);
  }

  public String getDescription()
  { return "sends testing messages via local server";
  }

  public void printHelp(PrintWriter out)
  { out.println(getDescription());
    out.println();
    out.println("usage:");
    out.println("> mail [properties_file]");
  }

  public void init(Server parent) throws ServiceException
  { // the parent server must be SMTPServer! 
    if (!(parent instanceof SMTPServer))
      throw new IllegalServiceClassException(SMTPServer.class);

    super.init(parent);

    // copy reference to SMTPServer - it will be used by sending threads
    server = (SMTPServer)parent;

    // an alternative and more universal way:
    //
    // MainServer main = Main.getMainServer();
    // server = (SMTPServer)main.getService("SMTP Server");
  }

  protected void execute(Subject subject, Service service, String[] args, Writer out) throws CommandException
  { try
    { // load settings from the property file - either default or provided via command argument
      String propname = "conf/samples/mail/mailclient.properties";
      if (args.length > 0)
        propname = args[0];
      Properties props = new Properties();
      props.load(new FileInputStream(new File(propname)));
  
      // create MIME message
      msg = new MimeMessageBuilder();
      msg.setFrom(props.getProperty("from"));
      msg.setTo(props.getProperty("to"));
      msg.setSubject(props.getProperty("subject"));
      String attachment = props.getProperty("attachment");
      if (attachment.equals("true"))
      { MimePartBuilder part = new MimePartBuilder();
        part.setName("Message Text");
        part.setContent(props.getProperty("text"));
        msg.addMimePart(part);
  
        part = new MimePartBuilder();
        String filename = props.getProperty("attachment-file-name");
        File file = new File(filename);
        part.setName(file.getName());
        String type = props.getProperty("attachment-mime-type", "application/octet-stream");
        String enc = props.getProperty("attachment-transfer-encoding", "base64");
        InputStream in = new FileInputStream(file);
        part.setContent(in, type, enc);
        in.close();
        msg.addMimePart(part);
        msg.setContentType("multipart/mixed");
      }
      else
        msg.setContent(props.getProperty("text"));

      // create and start sending threads
      int clients = Integer.parseInt(props.getProperty("clients"));
      int messages = Integer.parseInt(props.getProperty("messages"));
      int pause = Integer.parseInt(props.getProperty("pause"));

      PrintWriter pw = new PrintWriter(out, true);
      pw.println("Starting " + clients + " clients, each sending " + messages + " messages");
      for (int i = 0; i < clients; i++)
        new Sender(messages, pause).start();
    }
    catch (Exception e)
    { throw new CommandException(e);
    }

  }

  private class Sender extends Thread
  { private int messages;
    private int pause;

    Sender(int messages, int pause)
    { this.messages = messages;
      this.pause = pause;
    }

    public void run()
    { while (true)
      { try
        { synchronized (msg)          // MimeMessageBuilder is not thread-safe!
          { server.sendMessage(msg);
          }
        }
        catch (Exception e)
        { log(LOG_ERROR, "Error while sending message", e);
        }
  
        if (messages-- == 0)
          break;
  
        synchronized (this)
        { try
          { wait(pause*1000);
          }
          catch (InterruptedException e)
          {
          }
        }
      }
    }
  }
}
